;/*******************************/;
;/* Demo DC Motor  For ET-EXP4  */;
;/* Files Name  : LAB9-EX1.ASM  */;
;/* CPU Control : MCS51 Family  */;
;/* Assembler   : SXA51.EXE     */;
;/*   (DAC Control DC Motor)    */;
;/*******************************/;
;
;/* Port Scankey Matrix 4x3
PORT_KEY    EQU     P0                  ; Port Scan Keyboard
;
;/* Port For Control DAC
PORT_DAC    EQU     P1                  ; DAC Output
;
;/* Port Control Motor
PORT_MTR    EQU     P2                  ; DC Motor Control

            ORG     20H
FLAG_BUF:   DS      1                   ; Flag Status
ROW:        DS      1                   ; Row Buffer
COLUM:      DS      1                   ; Colum Buffer
SPEED:      DS      1                   ; Speed Motor
STACK:      DS      32
;                                       ; Stack 32 Byte
KEY_PRES    EQU     FLAG_BUF.0
KEY_STAS    EQU     FLAG_BUF.1

            ORG     0000H               ; Start Monitor
MAIN:       MOV     SP,#STACK           ; Stack = 32 Byte Internal RAM
            MOV     PORT_MTR,#0FFH
            ;
            SETB    PORT_MTR.4          ; Direction Motor
            CLR     PORT_MTR.5
            MOV     PORT_DAC,#0FFH      ; Start Motor
            MOV     R2,#250
STRT_MTR:   MOV     R3,#0FFH
            DJNZ    R3,$
            DJNZ    R2,STRT_MTR
            ;
            MOV     FLAG_BUF,#00H
            MOV     SPEED,#7FH          ; Speed 50%
            MOV     PORT_DAC,SPEED
            ;
WAIT_KEY:   LCALL   SCANKEY             ; Scankey 4X3
            CJNE    A,#0FFH,UP_KEY
            SJMP    WAIT_KEY            ; Loop Until Keypress
            ;
UP_KEY:     CJNE    A,#0AH,DN_KEY
            MOV     A,SPEED
            CJNE    A,#192,UP_SPEED     ; Speed 75%
            SJMP    WAIT_KEY
            ;
UP_SPEED:   INC     SPEED
            MOV     PORT_DAC,SPEED
            SJMP    WAIT_KEY
            ;
DN_KEY:     CJNE    A,#0BH,WAIT_KEY
            MOV     A,SPEED
            CJNE    A,#64,DN_SPEED      ; Speed 25%
            SJMP    WAIT_KEY
            ;
DN_SPEED:   DEC     SPEED
            MOV     PORT_DAC,SPEED
            SJMP    WAIT_KEY

;/********************************/;
;/*  Scan Keyboard Matrix  (4x3) */;
;/*  Output    : ACC (code 0..F) */;
;/*            : FFH (Not press) */;
;/*  Register  : ACC,B,R2,DPTR   */;
;/*  Sub. Call : KEY_DLY         */;
;/*            : KEY_NEW         */;
;/********************************/;
;
SCANKEY:    MOV     ROW,#3              ; Counter for Scan 4x3
            MOV     COLUM,#11111110B    ; Data for Scan Colum
SCAN1:      MOV     PORT_KEY,COLUM      ; Out Colum (P0-P2)
            MOV     A,PORT_KEY          ; In Row (P4-P7)
            ANL     A,#0F0H
            CJNE    A,#0F0H,PRESS       ; Check row only (PC4-PC7)
            MOV     A,COLUM             ; Scan Next Colum
            RL      A
            MOV     COLUM,A
            DJNZ    ROW,SCAN1
            MOV     A,#0FFH             ; Not Keypress
            CLR     KEY_PRES            ; not key Press
            SJMP    END_SCAN
            ;
PRESS:      LCALL   KEY_DLY             ; Debounce key
            MOV     PORT_KEY,COLUM      ; Out Colum (P0-P2)
            MOV     A,PORT_KEY          ; In Row (P4-P7)
            ANL     A,#0F0H
            CJNE    A,#0F0H,KEY_CODE    ; Row (PC4-PC7)
            SJMP    SCANKEY             ; No keybounce or noise
KEY_CODE:   LCALL   KEY_NEW
            JNB     KEY_STAS,SCAN1      ; if old key don't care
            MOV     B,A                 ; Row code (High)
            MOV     A,COLUM
            ANL     A,#0FH              ; Colum (Low Byte)
            ORL     A,B                 ; Mix Row & Colum
            MOV     B,A                 ; Row & Colum code
            ;
            MOV     DPTR,#KEY_TAB
            MOV     R2,#0FFH
            MOV     ROW,#12             ; 12 Key Maximum
CODE1:      INC     R2
            MOV     A,R2
            MOVC    A,@A+DPTR
            CJNE    A,B,CODE2
            MOV     A,R2                ; Number code 0..F
            SJMP    END_SCAN
            ;
CODE2:      DJNZ    ROW,CODE1
            MOV     A,#0FFH             ; Not Match Code
            CLR     KEY_PRES            ; not key Press
END_SCAN:   RET

;/**********************************/;
;/*     Check Old & New key        */;
;/*  KEY_STAS = 0 is old press Key */;
;/*  KEY_STAS = 1 is 1st press Key */;
;/**********************************/;
;
KEY_NEW:    CLR     KEY_STAS            ; Next time = old key
            JB      KEY_PRES,END_NEW    ; Jump if old key press
            SETB    KEY_PRES            ; Next time = old press
            SETB    KEY_STAS            ; Next time = old key
END_NEW:    RET

;/****************/;
;/*   Delay For  */;
;/* Debounce key */;
;/****************/;
;
KEY_DLY:    MOV     R6,#01FH
KEY_DL1:    MOV     R7,#0FFH
            DJNZ    R7,$
            DJNZ    R6,KEY_DL1
            RET

;/*********************/;
;/* Table of Key Code */;
;/*********************/;
;
KEY_TAB:    DB      07DH,0EEH,0EDH,0EBH     ; 0 1 2 3
            DB      0DEH,0DDH,0DBH,0BEH     ; 4 5 6 7
            DB      0BDH,0BBH,07EH,07BH     ; 8 9 * #

            END
