;/********************************/;
;/* Demo EEPROM 9346 For ET-EXP4 */;
;/* Files Name  : LAB7-EX2.ASM   */;
;/* CPU Control : MCS51 Family   */;
;/* Assembler   : SXA51.EXE      */;
;/*    (Write & Read to LCD)     */;
;/********************************/;
;
PORT_IO     EQU     P0                  ; Port Interface Serial I/O
CS_EEP      EQU     PORT_IO.0           ; CS EEP = PX.0
CS_ADC      EQU     PORT_IO.1           ; CS ADC = PX.1
DO_EEP      EQU     PORT_IO.2           ; Data Output
DI_EEP      EQU     PORT_IO.3           ; Data Input
SK_EEP      EQU     PORT_IO.4           ; Clock Synchronous
;
;/* Port For Control LCD (Port-P1)
PORT_LCD    EQU     P1                  ; Port Data LCD
RS_LCD      EQU     PORT_LCD.0          ; RS Select
E_LCD       EQU     PORT_LCD.2          ; Enable LCD

            ORG     20H
CNT_BUFF:   DS      1
STACK:      DS      32

            ORG     0000H               ; Start at Monitor
START:      MOV     SP,#STACK           ; Stack = 32 Byte
            CLR     E_LCD               ; Disable LCD
            CLR     CS_EEP              ; Stop EEP
            SETB    CS_ADC              ; Stop ADC
            ;
            MOV     R2,#0FFH            ; Power-on Delay
PWR_DLY:    MOV     R3,#0FFH
            DJNZ    R3,$
            DJNZ    R2,PWR_DLY
            LCALL   READ_EEP            ; Makesure Interface
            LCALL   INIT_LCD            ; Initial LCD4 Bit (16X2)
            ;
            MOV     A,#0                ; 1st Location Line1
            LCALL   GOTO_LCD
            LCALL   PRINT_LCD
            DB      'DEMO EEPROM 9346'  ; Message on Line-1
            DB      00H
            MOV     A,#40H              ; 1st Location Line2
            LCALL   GOTO_LCD
            LCALL   PRINT_LCD
            DB      'Read0-7:        '  ; Message on Line-2
            DB      00H
            ;
            LCALL   EWEN_EEP            ; Enable Erase/Write
            MOV     DPTR,#TAB_DATA      ; Data
            MOV     R0,#0               ; Address
            MOV     CNT_BUFF,#8
            ;
WRITE_DAT:  CLR     A
            MOVC    A,@A+DPTR
            MOV     B,A                 ; Data
            MOV     A,R0                ; Address
            LCALL   WRITE_EEP
            INC     DPTR                ; Next Data
            INC     R0
            DJNZ    CNT_BUFF,WRITE_DAT
            LCALL   EWDS_EEP            ; Disable Erase/Write
            ;
            MOV     A,#48H              ; Display EEPROM
            LCALL   GOTO_LCD
            MOV     CNT_BUFF,#8         ; 8 Byte Read
            MOV     R0,#0
READ_DAT:   MOV     A,R0                ; Get Address
            LCALL   READ_EEP
            LCALL   WR_LCD              ; Display Read
            INC     R0                  ; Next Read
            DJNZ    CNT_BUFF,READ_DAT
            ;
            SJMP    $

TAB_DATA:   DB      'EEP 8Bit'          ; Data to Test Write


;/**************************/;
;/* - EWEN Command (8-Bit) */;
;/* Format : 10 & 011XXXXX */;
;/*  (Eeprom Write Enable) */;
;/**************************/;
;
EWEN_EEP:   CLR     DI_EEP
            SETB    CS_EEP              ; Erase/Write Enable command
            SETB    DI_EEP
            LCALL   CLOCK               ; 1
            CLR     DI_EEP
            LCALL   CLOCK               ; 0
            LCALL   CLOCK               ; 0
            SETB    DI_EEP
            LCALL   CLOCK               ; 1
            LCALL   CLOCK               ; 1
            CLR     DI_EEP
            LCALL   CLOCK               ; X(0)
            LCALL   CLOCK               ; X(0)
            LCALL   CLOCK               ; X(0)
            LCALL   CLOCK               ; X(0)
            LCALL   CLOCK               ; X(0)
            CLR     CS_EEP              ; Standby
            RET

;/**************************/;
;/* - EWDS Command (8-Bit) */;
;/* Format : 10 & 000XXXXX */;
;/* (Eeprom Write Disable) */;
;/**************************/;
;
EWDS_EEP:   CLR     DI_EEP
            SETB    CS_EEP              ; Erase/Write Disable command
            SETB    DI_EEP
            LCALL   CLOCK               ; 1
            CLR     DI_EEP
            LCALL   CLOCK               ; 0
            LCALL   CLOCK               ; 0
            LCALL   CLOCK               ; 0
            LCALL   CLOCK               ; 0
            LCALL   CLOCK               ; X(0)
            LCALL   CLOCK               ; X(0)
            LCALL   CLOCK               ; X(0)
            LCALL   CLOCK               ; X(0)
            LCALL   CLOCK               ; X(0)
            CLR     CS_EEP              ; Standby
            RET

;/**************************/;
;/* - WRITE Command(8-Bit) */;
;/* Input  : ACC (Addr)    */;
;/*        : B   (Data)    */;
;/* Format : 10 & 1,A6..A0 */;
;/*  (Write Data By Addr)  */;
;/**************************/;
;
WRITE_EEP:  CLR     DI_EEP
            SETB    CS_EEP              ; Write Command
            SETB    DI_EEP
            LCALL   CLOCK               ; 1
            CLR     DI_EEP
            LCALL   CLOCK               ; 0
            SETB    DI_EEP
            LCALL   CLOCK               ; 1
            RL      A                   ; Shift A6 to A7
            MOV     R2,#7
            CLR     CY
WRITE_EE1:  RLC     A                   ; Write  Address 7 bit
            MOV     DI_EEP,C
            LCALL   CLOCK
            DJNZ    R2,WRITE_EE1        ; Repeat Write address
            ;
            MOV     R2,#8               ; Write data 8 bit in B
            MOV     A,B
WRITE_EE2:  RLC     A                   ; Write data D7..D0
            MOV     DI_EEP,C
            LCALL   CLOCK
            DJNZ    R2,WRITE_EE2
            ;
            CLR     DI_EEP
            CLR     CS_EEP
            LCALL   CLOCK               ; TCSL
            SETB    CS_EEP
WRITE_EE3:  LCALL   CLOCK               ; Wait Ready State
            SETB    DO_EEP
            JNB     DO_EEP,WRITE_EE3    ; Repeat if Busy
            CLR     CS_EEP              ; OK Ready So Standby
            RET

;/**************************/;
;/* - ERASE Command(8-Bit) */;
;/* Input  : ACC (Addr)    */;
;/* Format : 11 & 1,A6..A0 */;
;/*   (Erase By Address)   */;
;/**************************/;
;
ERASE_EEP:  CLR     DI_EEP
            SETB    CS_EEP              ; Erase command
            SETB    DI_EEP
            LCALL   CLOCK               ; 1
            LCALL   CLOCK               ; 1
            LCALL   CLOCK               ; 1
            RL      A                   ; Shift A6 to A7
            MOV     R2,#7
            CLR     CY
ERASE_EE1:  RLC     A                   ; Write Address 7 bit
            MOV     DI_EEP,C
            LCALL   CLOCK
            DJNZ    R2,ERASE_EE1
            CLR     DI_EEP
            CLR     CS_EEP
            LCALL   CLOCK               ; TCLS
            SETB    CS_EEP
ERASE_EE2:  LCALL   CLOCK               ; Wait Ready
            SETB    DO_EEP
            JNB     DO_EEP,ERASE_EE2    ; Loop if Busy
            CLR     CS_EEP              ; OK Ready So Standby
            RET

;/**************************/;
;/* - READ Command (8-Bit) */;
;/* Input  : ACC (Addr)    */;
;/* Output : ACC (Data)    */;
;/* Format : 11 & 0,A6..A0 */;
;/*   (Read Data 1-Byte)   */;
;/**************************/;
;
READ_EEP:   CLR     DI_EEP
            SETB    CS_EEP              ; Read Command
            SETB    DI_EEP
            LCALL   CLOCK               ; 1
            LCALL   CLOCK               ; 1
            CLR     DI_EEP
            LCALL   CLOCK               ; 0
            RL      A                   ; Shift A6 to A7
            MOV     R2,#7
            CLR     CY
READ_EE1:   RLC     A                   ; Write address 7 bit
            MOV     DI_EEP,C
            LCALL   CLOCK
            DJNZ    R2,READ_EE1         ; Repeat write address
            CLR     DI_EEP
            CLR     A                   ; Clear Output Buffer
            MOV     R2,#8
READ_EE2:   LCALL   CLOCK               ; Read data D0..D7
            SETB    DO_EEP
            MOV     C,DO_EEP            ; Read data out bit
            RLC     A
            DJNZ    R2,READ_EE2
            CLR     CS_EEP              ; Standby
            RET

;/******************/;
;/*  Serial Clock  */;
;/******************/;
;
CLOCK:      SETB    SK_EEP              ; SK = Hi
            NOP
            NOP
            NOP
            NOP
            CLR     SK_EEP              ; SK = Low
            NOP
            NOP
            NOP
            NOP
            RET

;/*******************/;
;/*   Initial LCD   */;
;/* 4-Bit Interface */;
;/* -> RS   = PX.0  */;
;/* -> E    = PX.2  */;
;/* -> D4-7 = PX4-7 */;
;/*******************/;
;
INIT_LCD:   MOV     A,#33H              ; DL = 1 ,DL =1
            LCALL   WR_INS
            MOV     A,#32H              ; DL = 1 ,DL =0
            LCALL   WR_INS
            MOV     A,#28H              ; Function set
            LCALL   WR_INS              ; DL=0 4Bit,N=1 2Line,F=0 5X7
            MOV     A,#0CH              ; Display on/off Control
            LCALL   WR_INS              ; Entry display,cursor off,not blink
            MOV     A,#06H              ; Entry mode set
            LCALL   WR_INS              ; I/D=1 Increment,S=0 cursor shift
            MOV     A,#01H              ; Clear Display
            LCALL   WR_INS              ; Clear Display,Set DD RAM addr=0
            RET

;/**********************/;
;/* Enable Pin E LCD   */;
;/*  Pin E LCD = PX.2  */;
;/* Active Chip select */;
;/**********************/;
;
EN_LCD:     PUSH    B
            MOV     B,#50               ; Busy Delay LCD
            DJNZ    B,$
            POP     B
            SETB    E_LCD               ; Enable LCD (E=PX.2=1)
            NOP
            CLR     E_LCD               ; Disable LCD (E=PX.2=0)
            RET

;/**************************/;
;/*   Write ASCII to LCD   */;
;/* Input  : ACC  (ASCII)  */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_LCD:     PUSH    ACC                 ; Save Data Low Byte
            ANL     A,#0F0H
            MOV     PORT_LCD,A          ; High Byte & RS=1 Write
            SETB    RS_LCD              ; Set RS For Write Data Select
            LCALL   EN_LCD
            POP     ACC                 ; Restore Data Low Byte
            SWAP    A                   ; Shift Low Byte to High Byte
            ANL     A,#0F0H
            MOV     PORT_LCD,A          ; Low Byte & RS=1 Write
            SETB    RS_LCD              ; Set RS For Write Data Select
            LCALL   EN_LCD
            RET

;/**************************/;
;/* Write Instruction LCD  */;
;/* Input  : ACC (Command) */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_INS:     PUSH    ACC                 ; Save Data Low Byte
            ANL     A,#0F0H
            MOV     PORT_LCD,A          ; High Byte & RS=0 Write
            LCALL   EN_LCD
            ;
            POP     ACC                 ; Restore Data Low Byte
            SWAP    A                   ; Shift Low Byte to High Byte
            ANL     A,#0F0H
            MOV     PORT_LCD,A          ; Low Byte & RS=0 Write
            LCALL   EN_LCD
            MOV     R2,#2               ; Busy Ins Delay
INS_DLY:    MOV     R3,#250
            DJNZ    R3,$
            DJNZ    R2,INS_DLY
            RET
            
;/************************/;
;/* Goto position of LCD */;
;/* Input : ACC (addr.)  */;
;/************************/;
;
GOTO_LCD:   SETB    ACC.7
            LCALL   WR_INS
            RET

;/************************************/;
;/*  Print Data data to Serial Port  */;
;/*  to display buffer.              */;
;/*  Usage    : LCALL PRINT_LCD      */;
;/*           : DB   'xxxx',00       */;
;/*  Register : ACC                  */;
;/*  Note     : last byte must be 00 */;
;/************************************/;
;
PRINT_LCD:  POP     DPH
            POP     DPL
PRINT1:     CLR     A
            MOVC    A,@A+DPTR
            CJNE    A,#00H,PRINT2
            SJMP    PRINT3
PRINT2:     LCALL   WR_LCD
            INC     DPTR
            SJMP    PRINT1
PRINT3:     PUSH    DPL
            PUSH    DPH
            RET

            END

