;/*******************************/;
;/* Demo DC-Motor For ET-EXP4   */;
;/* Files Name  : LAB9-EX1.ASM  */;
;/* CPU Control : 68HC811E2     */;
;/* Assembler   : AS11NEW.EXE   */;
;/*   (Used DAC Control Motor)  */;
;/*******************************/;
;
;/* Port Scankey Matrix 4x3 (Port-C)
PORT_KEY    EQU     $1003               ; Port-C
CTRL_KEY    EQU     $1007               ; DDRC 0=In,1=Out
;
;/* Port For Control DAC
PORT_DAC    EQU     $1004               ; DAC Output(Port-B)
;
;/* Port Control Motor
PORT_MTR    EQU     $1000               ; DC Motor Control(Port-A)
PACTL       EQU     $1026               ; Direction Port-A

            ORG     $00
SCAN_BUF    RMB     1                   ; Colum Scan Control
KEY_FLAG    RMB     1                   ; 1 = old ,0 = new
SPEED       RMB     1                   ; Speed Motor

            ORG     $F800               ; Start Monitor of 68H1811E2
START       LDS     #$FF                ; Stack on Last Internal RAM
            LDAA    #%10001000
            STAA    PACTL               ; PA3,PA7 = Output Port
            LDAA    #%00010000          
            STAA    PORT_MTR            ; PA4 = "1" and PA5 = "0"
            CLR     KEY_FLAG
            LDAA    #%00001111          ; PCH=In,PCL=Out
            STAA    CTRL_KEY
            ;
            LDAA    #$FF                ; Start Motor
            STAA    PORT_DAC
            ;
            LDX     #$FFFF              ; Power-on Delay
PWR_DLY     DEX
            BNE     PWR_DLY
            ;
            LDAA    #$7F                ; Speed 50%
            STAA    SPEED
            STAA    PORT_DAC
            ;
WAIT_KEY    JSR     SCAN_KEY            ; Scankey 4X3
            CMPA    #$FF
            BNE     UP_KEY
            BRA     WAIT_KEY            ; Loop Until Keypress
            ;
UP_KEY      CMPA    #$0A
            BNE     DN_KEY
            LDAA    SPEED
            CMPA    #192
            BLS     UP_SPEED            ; Speed 75%
            BRA     WAIT_KEY            ; Over 75%
            ;
UP_SPEED    INC     SPEED
            LDAA    SPEED
            STAA    PORT_DAC
            BRA     WAIT_KEY
            ;
DN_KEY      CMPA    #$0B
            BNE     WAIT_KEY
            LDAA    SPEED
            CMPA    #64
            BHS     DN_SPEED            ; Speed 25%
            BRA     WAIT_KEY            ; Lower 25%
            ;
DN_SPEED    DEC     SPEED
            LDAA    SPEED
            STAA    PORT_DAC
            BRA     WAIT_KEY

;/******************************/;
;/* Scan Keyboard Matrix (4x3) */;
;/* PC0..PC3 : Colum (Out)     */;
;/* PC4..PC7 : Row (In)        */;
;/* Output   : ACC (Code 0..B) */;
;/*          : FFH (Not Press) */;
;/******************************/;
;
SCAN_KEY    LDAB    #3                  ; Counter for Scan 4x3
            LDAA    #%00000001          ; Data For Colum Scan
            STAA    SCAN_BUF
            ;
SCAN1       LDAA    SCAN_BUF
            COMA
            STAA    PORT_KEY            ; Scan Colum (PC0..PC3)
            LDAA    PORT_KEY            ; In Row (PC4..PC7)
            ANDA    #$F0
            CMPA    #$F0                ; Check row only (PC0..PC3)
            BNE     KEY_PRES            ; Jump if keypress
            LDAA    SCAN_BUF
            ASLA
            STAA    SCAN_BUF            ; Next Colum Scan
            DECB
            BNE     SCAN1
            LDAA    #$FF                ; Not Keypress
            CLR     KEY_FLAG            ; Reset Press
            BRA     END_SCAN
            ;
KEY_PRES    JSR     DLY_10MS            ; Debounce key
            LDAA    PORT_KEY            ; In Row (PC4..PC7)
            ANDA    #$F0
            CMPA    #$F0                ; Check row only (PC4..PC7)
            BNE     KEY_CODE            ; YES keypress
            BRA     SCAN_KEY            ; NO keybounce or noise
            ;
KEY_CODE    JSR     KEY_NEW
            BCC     SCAN1               ; Ignore if old key
            LDAB    SCAN_BUF
            ABA                         ; Mix Row & Colum
            ;
            LDY     #KEY_TAB            ; Row & Colum Code
            LDAB    #12                 ; 12 Key Count
CODE1       CMPA    0,Y
            BEQ     CODE2
            INY
            DECB
            BNE     CODE1
            LDAA    #$FF                ; Not Math in 12 Key
            BRA     END_SCAN
CODE2       DECB
            TBA                         ; Keycode (0..B)
END_SCAN    RTS

;/******************************/;
;/* Check Old & New key Status */;
;/* Out Status = Carry flag    */;
;/* Carry Flag(0) is old press */;
;/* Carry Flag(1) is new press */;
;/******************************/;
;
KEY_NEW     PSHA
            CLC                         ; old key Status
            LDAA    KEY_FLAG
            BNE     END_NEW             ; Not Zero
            INCA                        ; old key in Next Time
            STAA    KEY_FLAG
            SEC                         ; New Key Status
END_NEW     PULA
            RTS

;/**************************/;
;/* Table Code For Key 4x3 */;
;/* 1(E1H)  2(E2H)  3(E4H) */;
;/* 4(D1H)  5(D2H)  6(D4H) */;
;/* 7(B1H)  8(B2H)  9(D4H) */;
;/* *(71H)  0(72H)  #(74H) */;
;/**************************/;
;
KEY_TAB     FCB     $74,$71,$B4,$B2     ; #    *    9    8
            FCB     $B1,$D4,$D2,$D1     ; 7    6    5    4
            FCB     $E4,$E2,$E1,$72     ; 3    2    1    0

;/*******************/;
;/* Delay Time 10mS */;
;/* E-Clock = 2 MHz */;
;/*******************/;
;
DLY_10MS    PSHX
            LDX     #$0D06              ; 10 mSec counter
DLY10       DEX
            BNE     DLY10
            PULX
            RTS

            ORG     $FFFE               ; Reset Vector
            FDB     START

            END

