;/*********************************/;
;/* Demo Internal ADC For ET-EXP4 */;
;/* Files Name  : LAB10EX1.ASM    */;
;/* CPU Control : 68HC811E2       */;
;/* Assembler   : AS11NEW.EXE     */;
;/*    (Display Volt ADC on LCD)  */;
;/*********************************/;
;
;/* Port For Control LCD (Port-B)
PORT_LCD    EQU     $1004               ; Port Data LCD (PortB)
ADCTL       EQU     $1030               ; ADC Control Register
ADR1        EQU     $1031               ; ADC Result Register1
ADR2        EQU     $1032               ; ADC Result Register2
ADR3        EQU     $1033               ; ADC Result Register3
ADR4        EQU     $1034               ; ADC Result Register4
OPTION      EQU     $1039
            ;
            ORG     $0000
HEX_BUFF    RMB     2
DEC_BUFF    RMB     5
            ;
            ORG     $F800               ; Start at Monitor 68HC811E2
START       LDS     #$FF                ; Stack = Last Internal RAM
            LDAA    #%10010011          ; Initial Option ADC
            STAA    OPTION
            LDX     #$FFFF              ; Power-on Delay
PWR_DLY     DEX
            BNE     PWR_DLY
            JSR     INIT_LCD            ; Initial LCD4 Bit (16X2)
            ;
            LDAA    #$00                ; 1st Location Line1
            JSR     GOTO_LCD
            JSR     PRINT_LCD
            FCC     'DEMO Internal AD'  ; Message on Line-1
            FCB     $00
            LDAA    #$40                ; 1st Location Line2
            JSR     GOTO_LCD
            JSR     PRINT_LCD
            FCC     'ADC CH0 =      V'  ; Message on Line-2
            FCB     $00
            ;
LOOP        LDAA    #$4AH               ; Display ADC Cursor
            JSR     GOTO_LCD
            JSR     MULT_ADC1           ; Convert ADC1..4
            LDAA    ADR1                ; Get Step ADC
            LDAB    #196                ; 5000mV / 255 = 19.6
            MUL                         ; A x B = D
            STD     HEX_BUFF
            JSR     HTOD
            LDAA    DEC_BUFF+0
            JSR     WR_LCD
            LDAA    #'.'
            JSR     WR_LCD
            LDAA    DEC_BUFF+1
            JSR     WR_LCD
            LDAA    DEC_BUFF+2
            JSR     WR_LCD
            LDAA    DEC_BUFF+3
            JSR     WR_LCD
            BRA     LOOP

;/*******************/;
;/*   Initial LCD   */;
;/* 4-Bit Interface */;
;/* -> RS   = PB0   */;
;/* -> E    = PB2   */;
;/* -> D4-7 = PB4-7 */;
;/*******************/;
;
INIT_LCD    LDAA    #$33                ; DL = 1 ,DL =1
            JSR     WR_INS
            LDAA    #$32                ; DL = 1 ,DL =0
            JSR     WR_INS
            LDAA    #$28                ; Function set
            JSR     WR_INS              ; DL=0 4Bit,N=1 2Line,F=0 5X7
            LDAA    #$0C                ; Display on/off Control
            JSR     WR_INS              ; Entry display,cursor off,not blink
            LDAA    #$06                ; Entry mode set
            JSR     WR_INS              ; I/D=1 Increment,S=0 cursor shift
            LDAA    #$01                ; Clear Display
            JSR     WR_INS              ; Clear Display,Set DD RAM addr=0
            RTS

;/**********************/;
;/* Enable Pin E LCD   */;
;/*  Pin E LCD = PB.2  */;
;/* Active Chip select */;
;/**********************/;
;
EN_LCD      PSHB
            LDAB    #20                 ; 100uS counter
BUSY        DECB
            BNE     BUSY
            PULB
            ORAA    #%00000100          ; Enable LCD (E=PB2=1)
            STAA    PORT_LCD
            ANDA    #%11111011          ; Disable LCD (E=PB2=0)
            STAA    PORT_LCD
            RTS

;/**************************/;
;/*   Write ASCII to LCD   */;
;/* Input  : ACC  (ASCII)  */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_LCD:     PSHA                        ; Save Data Low Byte
            ANDA    #$F0
            ORAA    #%00000001          ; Set RS For Write Data Select
            STAA    PORT_LCD            ; High Byte & RS=1 Write
            JSR     EN_LCD
            PULA                        ; Restore Data Low Byte
            ASLA                        ; Shift Low Byte to High Byte
            ASLA
            ASLA
            ASLA
            ANDA    #$F0
            ORAA    #%00000001          ; Set RS For Write Data Select
            STAA    PORT_LCD            ; Low Byte & RS=1 Write
            JSR     EN_LCD
            RTS

;/**************************/;
;/* Write Instruction LCD  */;
;/* Input  : ACC (Command) */;
;/* Output : Data bus LCD  */;
;/**************************/;
;
WR_INS      PSHA                        ; Save Data Low Byte
            ANDA    #$F0
            STAA    PORT_LCD            ; High Byte & RS=0 Write
            JSR     EN_LCD
            PULA                        ; Restore Data Low Byte
            ASLA                        ; Shift Low Byte to High Byte
            ASLA
            ASLA
            ASLA
            ANDA    #$F0
            STAA    PORT_LCD            ; Low Byte & RS=0 Write
            JSR     EN_LCD
            PSHX
            LDX     #$0632              ; 5 mSec counter
INS_DLY     DEX
            BNE     INS_DLY
            PULX
            RTS
            
;/************************/;
;/* Goto position of LCD */;
;/* Input : ACC (addr.)  */;
;/************************/;
;
GOTO_LCD    ORAA    #$80                ; Set ACC.7
            JSR     WR_INS
            RTS

;/********************************/;
;/* Print ASCII data to LCD Port */;
;/* Usage    : JSR  PRINT_LCD    */;
;/*          : FCC  'xxxxxxxx'   */;
;/*          : FCB  $00          */;
;/* Note     : last byte = $00   */;
;/********************************/;
;
PRINT_LCD   PULX                        ; Pop Stack
            PSHA
            PSHB
PRINT1      LDAA    0,X                 ; Get Data For Print
            CMPA    #$00                ; Check Last Byte
            BNE     PRINT2              ; Jump if not Last Byte
            BRA     PRINT3
PRINT2      JSR     WR_LCD              ; Write Data to LCD
            INX
            BRA     PRINT1
PRINT3      INX
            PULB
            PULA
            PSHX                        ; Push Stack
            RTS

;/*********************************/;
;/* Multiple 4 Channel Low  (LSB) */;
;/* Output   : ADR1 (Ch1 Result)  */;
;/*          : ADR2 (Ch2 Result)  */;
;/*          : ADR3 (Ch3 Result)  */;
;/*          : ADR4 (Ch4 Result)  */;
;/*********************************/;
;
MULT_ADC1   PSHA
            LDAA    #%00010000              ; Multiple 4 Channel LSB
            STAA    ADCTL
CONVERT1    LDAA    ADCTL
            ANDA    #$80                    ; Convertion Complete Flag
            BEQ     CONVERT1                ; Wait ADC Convert Ready
            PULA
            RTS

;/*********************************/;
;/* Multiple 4 Channel High (MSB) */;
;/* Output    : ADR1 (Ch5 Result) */;
;/*           : ADR2 (Ch6 Result) */;
;/*           : ADR3 (Ch7 Result) */;
;/*           : ADR4 (Ch8 Result) */;
;/*********************************/;
;
MULT_ADC2   PSHA
            LDAA    #%00010100              ; Multiple 4 Channel MSB
            STAA    ADCTL
CONVERT2    LDAA    ADCTL
            ANDA    #$80                    ; Convertion Complete Flag
            BEQ     CONVERT2                ; Wait ADC Convert Ready
            PULA
            RTS

;/**************************/;
;/* Convert HEX to Decimal */;
;/* Input  : HEX_BUFF      */;
;/* Output : DEC_BUFF      */;
;/**************************/;
;
HTOD        PSHX                            ; Save-IX
            LDD     HEX_BUFF
            LDX     #10000
            IDIV
            XGDX
            ADDB    #$30
            STAB    DEC_BUFF+0
            XGDX
            LDX     #1000
            IDIV
            XGDX
            ADDB    #$30
            STAB    DEC_BUFF+1
            XGDX
            LDX     #100
            IDIV
            XGDX
            ADDB    #$30
            STAB    DEC_BUFF+2
            XGDX
            LDX     #10
            IDIV
            ADDB    #$30
            STAB    DEC_BUFF+4
            XGDX
            ADDB    #$30
            STAB    DEC_BUFF+3
            PULX                            ; Restore-IX
            RTS

            ORG     $FFFE                   ; Reset Vector
            FDB     START

            END

