;/*********************************/;
;/* Demo Timer 68HC11 For ET-EXP4 */;
;/* Files Name  : LAB11EX3.ASM    */;
;/* CPU Control : 68HC811E2       */;
;/* Assembler   : AS11NEW.EXE     */;
;/* (Adjust PWM 2%...98% on PA6)  */;
;/*********************************/;
;
;/* Timer Register Address
REGBAS      EQU     $1000                   ; Register base
OC1M        EQU     $0C                     ; OC1 action mask register
OC1D        EQU     $0D                     ; OC1 action data register
TCNT        EQU     $0E                     ; Timer counter register
TOC1        EQU     $16                     ; Output compare 1 register
TOC2        EQU     $18                     ; Output compare 2 register
TOC3        EQU     $1A                     ; Output compare 3 register
TCTL1       EQU     $20                     ; Timer control register 1
TMSK1       EQU     $22                     ; Timer interrupt mask register 1
TFLG1       EQU     $23                     ; Timer interrupt flag register 1
            ;
            ORG     $0000                   ; Internal RAM
PWMP1P      RMB     1                       ; % PWM Buffer (1-256 Cycle)
PWMDC1      RMB     1                       ; % Duty Cycle of PWM OC2
OFFLO       RMB     2                       ; OC2 High Pulse Time Offset
OFFHI       RMB     2                       ; OC2 Low Pulse Time Offset
            ;
            ORG     $F800                   ; Start of Main Program
MAIN        LDS     #$FF                    ; Stack Internal RAM
            LDX     #REGBAS                 ; Base Register Index
            ;
            LDAA    #200                    ; Defualt = 100Hz Period Time
            STAA    PWMP1P                  ; Set PWM = 200 Cycle (200x0.5uS)
            ;
            LDAA    #%11000000              ; OM2:OL2 = 1:1 Set PA6 = High
            STAA    TCTL1,X
            LDD     #$0000                  ; Start PWM Cycle = 0000
            STD     TOC2,X
            LDAA    #%01000000              ; Clear OC2F Flag
            STAA    TFLG1,X
            STAA    TMSK1,X                 ; Enable OC2 Interupt
            CLI                             ; Enable Global Interupt
            ;
            ;/* Generate PWM on OC2(PA6) */;
STRT_PWM    LDAA    #2                      ; Start from %PWM from 2..98%
            STAA    PWMDC1                  ; Start PWM Duty Cycle = 2%
            ;
INC_PWM     JSR     UPDATE_PWM              ; Update PWM
            JSR     DELAY                   ; Delay Output PWM Signal
            ;
            INC     PWMDC1
            LDAA    PWMDC1
            CMPA    #98                     ; Maximum Duty Cycle ?
            BNE     INC_PWM                 ; Loop Gen. PWM Until 98%
            BRA     STRT_PWM                ; PWM = 98% Restart at 2%

;/*******************/;
;/* Update PWM Time */;
;/*******************/;
;
UPDATE_PWM  LDAA    PWMDC1                  ; OC2 High Pulse Time
            LDAB    PWMP1P                  ; % PWM Cycle
            MUL
            STD     OFFHI                   ;/* Save High Pulse Cycle Time */;
            ;
            LDAA    PWMP1P                  ; % PWM Cycle
            LDAB    #100                    ; 100 % PWM Cycle (Total Pulse)
            MUL
            SUBD    OFFHI                   ; 100% Pulse - High Pulse = Low Pulse
            STD     OFFLO                   ;/* Save Low Pulse Cycle Time */;
            RTS

;/*********************/;
;/* Delay Time 100 mS */;
;/*********************/;
;
DELAY       PSHB                        ; Save Register
            PSHX
            LDAB    #10                 ; 10mS x 10 = 100 mS
DLY1        LDX     #$0D06              ; 10mS Count
DLY2        DEX
            BNE     DLY2
            DECB
            BNE     DLY1
            PULX                        ; Restore Register
            PULB
            RTS

;/*************************/;
;/* OC2 Overflow Interupt */;
;/* Update New TOC2 Count */;
;/*   "Toggle PA6 Pin"    */;
;/*************************/;
;
OC2_INT     LDX     #REGBAS                 ; Base Register Index
            BRCLR   TCTL1,X $40 ADDLO       ; If OL2 Clear = Low Pulse Cycle
            LDD     OFFHI                   ; If OL2 Set = High Pulse Cycle
            BRA     UPOC2
ADDLO       LDD     OFFLO
UPOC2       ADDD    TOC2,X                  ; New Cycle Count + Last Count
            STD     TOC2,X                  ; Update Next Time OC2
            LDAA    TCTL1,X
            EORA    #%01000000              ; Toggle OL2 Bit
            STAA    TCTL1,X                 ; OM2:OL2 = 1:X
            BCLR    TFLG1,X %10111111       ; Clear OC2F Flag
            RTI

            ORG     $FFE6
            FDB     OC2_INT                 ; OC2 Interupt Service
            ;
            ORG     $FFFE
            FDB     MAIN                    ; Reset Vector

            END
